unit U_IntegerPartition2;

{$MODE Delphi}

 {Copyright  © 2005-2013, Gary Darby,  www.DelphiForFun.org
 This program may be used or modified for any non-commercial purpose
 so long as this original notice remains in place.
 All other rights are reserved
 }

{A test program for the TIntPartition class.  Displays all integer partitions
 or those of a specified size for a given integer.  Also displays partition
 counts }


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, ComCtrls, shellAPI, ExtCtrls, strutils,  UIntegerpartition;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    GenBtn: TButton;
    Memo1: TMemo;
    NbrSpin: TSpinEdit;
    Label2: TLabel;
    StaticText1: TStaticText;
    Panel1: TPanel;
    RankBtn: TButton;
    NSpin: TSpinEdit;
    PartSizeSpin: TSpinEdit;
    TypeRGrp: TRadioGroup;
    Label4: TLabel;
    Results: TMemo;
    RankEdt: TEdit;
    Timer1: TTimer;
    ProgressBar1: TProgressBar;
    procedure GenBtnClick(Sender: TObject);
    procedure StaticText1Click(Sender: TObject);
    //procedure CountBtnClick(Sender: TObject);
    procedure RankBtnClick(Sender: TObject);
    procedure NSpinExit(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  public
    count:integer;
    target:int64;
    progress:int64; {Used to help track progress of unrank calculation}
    function Handlepartition:boolean;
    function CheckStopFlag:boolean;
  end;

var
  Form1: TForm1;

implementation
{$R *.lfm}

uses math;

procedure TForm1.FormActivate(Sender: TObject);
begin
  nspinExit(sender);
end;


{********************** GenBtnClick *********}
procedure TForm1.GenBtnClick(Sender: TObject);
{Generate all or a subset of partitions}
var
  N,K:integer;
begin

  Results.clear;
  N:=NSpin.Value; {the setsize}
  K:=PartSizeSpin.value;

  count:=0;
  with   defpartition,  {the default pre-created partition class instance}
         results, lines do
  Case TypeRgrp.ItemIndex of  {what to generate}
    0: {generate all}
    begin
      pcount:=defpartition.partitioncount(n,0);
      add(format('There are %19.0n total partitions of %d',[0.0+pcount,n]));
      add(format('%d Displayed',[min(pcount, NbrSpin.value)]));
      add('-------------------------------------');
      {Calling with partition size of K=0 generates all}
      {Handlepartiton is the routine called by partitionInit with each partition generated}
      partitioninit(N, 0{= generate all}, HandlePartition);
    end;
    1: {Generate all of length K}
    begin
      pcount:=defpartition.partitioncount(n,k);
      Add(format('There are %.0n partitions of %d with %d elements',
                               [0.0+pcount,n,k]));
      add(format('%d Displayed',[min(pcount, NbrSpin.value)]));
      add('-------------------------------------');
      partitioninit(N,K,HandlePartition);
    end;
    2: {generate all with maxval K}
    Begin
      pcount:=defpartition.partitioncount(n,k);
      add(format('There are %.0n partitions of %d with maximum value %d',
                               [0.0+pcount,n,K]));
      add(format('%d Displayed',[min(pcount, NbrSpin.value)]));
      add('-------------------------------------');
      partitionmaxval(N,K,0,Handlepartition);
    end;

  end; {case}
  results.selstart:=0;    {scroll display back to top}
  results.sellength:=0;
end;




{************* Handlepartition ***********}
function TForm1.handlepartition:boolean;
{Procedure called for each partition returned}
var  j:integer;
     s,x:string;

begin
  with defpartition do
  begin
    s:='{';
    for j:=0 to partsize-1 do {for each element of this subset}
    begin {build the entry to display }
      if p[j]>=0 then s:=s+inttostr(p[j])+','
      else break;
    end;
    {fix up some punctuation}
    x:=rightstr(s,1);
    if x=',' then s:=stuffstring(s,length(s),1,'}');
    if rightstr(s,1) ='}' then   s:=s+',{';
    if rightstr(s,1)='{' then delete(s,length(s)-1,2);
    inc(count);
    Results.lines.add(s); {display the partition}
  end;
  result:= count<NbrSpin.value;
end;


{******** NSpinExit ***********}
procedure TForm1.NSpinExit(Sender: TObject);
{Set maximum partition size to the N, number being partitoined}
begin
  partsizespin.maxvalue:=nspin.value
end;


{************ UnRankBtnClick ***********}
procedure TForm1.RankBtnClick(Sender: TObject);
{Unrank returns the partition corresponding to a given rank}
var
  n:integer;
  i,k:integer;
  rank,sum:int64;
  pp1:NTableArray;
  s:string;
begin
  with TButton(sender) do
  if caption='Stop'
  then  tag:=1
  else
  with defpartition, results.lines  do
  begin
    Results.Clear;
    {Test P(n,k)=P(n-1,k-1) + P(n-k,k)}

    s:=stringreplace(rankedt.text,',','',[rfreplaceall]);
    rank:=strtoint64def(s,0)-1;
    n:=nspin.value;
    partitioncount(n,0);
    if (rank>=0) and (rank<pcount) then
    begin
      TypeRGrp.itemindex:=0;
      {Results.lines.}add(format('Partition with rank %.0n in list of all partitions for integer %d',
                    [1.0+rank, NSpin.value]));

      setlength(pp1,N+1,N+1);
      NPart_Table(N,N,pp1); {build the NPartTable of partition lengths N by maxval}
      sum:=0;
      k:=0;
      for i:=high(pp1[N]) downto 0 do
      begin {sum all of the preceeding partition counts for maxvals< maxval of "rank"}
        if (sum+pp1[n,i] >rank) then
        begin  {we have found the maxval which contains the requested rank}
          k:=i; {save it in K}
          break;
        end
        else inc(sum,pp1[n,i]);
      end;
      target:=rank-sum+1; {now the target is this far into the partitions with
                            maxval of "rank"}
      //if target>100000 then
      begin
        screen.cursor:=crhourglass;
        rankbtn.Caption:='Stop';
        rankbtn.tag:=0;
        progress:=0;
      end;
      partitionmaxval(N,k,target, CheckStopFlag); {go find that partition}
      if rankbtn.tag=0 then handlepartition
      else showmessage('Search interrupted');
    end
    else showmessage(format('Invalid rank, must be integer > 0 and <=%d',[pcount]));
  end;
  screen.cursor:=crdefault;
  rankbtn.Caption:='Show the partition with rank=';
  progressbar1.position:=100;
end;

function TForm1.CheckStopFlag:boolean;
begin
  with defpartition do
  begin
    inc(progress);
    result:=true;
    if progress=target then result:=false
    else
    if (progress and $3fffffff)>0 then{update progress bar once in a while}
    begin
      progressbar1.position:= 100*progress div target;
      application.processmessages;
      if RankBtn.tag=1 then result:=false;
    end;
  end;
end;

procedure TForm1.StaticText1Click(Sender: TObject);
begin
   ShellExecute(Handle, 'open', 'http://www.delphiforfun.org/',
               nil, nil, SW_SHOWNORMAL);
end;
end.
